@@ -20,7 +20,8 @@
 #include <xtl/xsequence.hpp>
 #include <xtl/xtype_traits.hpp>
 
-#include "xexpression.hpp"
+#include "xaccessible.hpp"
+#include "xexpression_traits.hpp"
 #include "xiterable.hpp"
 #include "xlayout.hpp"
 #include "xscalar.hpp"
@@ -40,47 +41,9 @@ namespace xt
     namespace detail
     {
 
-        /********************
-         * common_size_type *
-         ********************/
-
-        template <class... Args>
-        struct common_size_type
-        {
-            using type = std::common_type_t<typename Args::size_type...>;
-        };
-
-        template <>
-        struct common_size_type<>
-        {
-            using type = std::size_t;
-        };
-
-        template <class... Args>
-        using common_size_type_t = typename common_size_type<Args...>::type;
-
         template <bool... B>
         using conjunction_c = xtl::conjunction<std::integral_constant<bool, B>...>;
 
-        /**************************
-         * common_difference type *
-         **************************/
-
-        template <class... Args>
-        struct common_difference_type
-        {
-            using type = std::common_type_t<typename Args::difference_type...>;
-        };
-
-        template <>
-        struct common_difference_type<>
-        {
-            using type = std::ptrdiff_t;
-        };
-
-        template <class... Args>
-        using common_difference_type_t = typename common_difference_type<Args...>::type;
-
         /************************
          * xfunction_cache_impl *
          ************************/
@@ -134,13 +97,7 @@ namespace xt
         {
         };
 
-        template <class T>
-        struct meta_identity
-        {
-            using type = T;
-        };
-
-        // This meta struct checks wether SIMD should be activated for our 
+        // This meta struct checks wether SIMD should be activated for our
         // functor "F"
         template <class V, class F, class... CT>
         struct xsimd_meta_getter
@@ -154,7 +111,7 @@ namespace xt
             using simd_value_type = xtl::mpl::eval_if_t<simd_arguments_exist,
                                                         meta_identity<xsimd::simd_type<scalar_result_type>>,
                                                         make_invalid_type<>>;
-            // if all types are supported, check that the functor has a working 
+            // if all types are supported, check that the functor has a working
             // simd_apply and all arguments have the simd interface
             using use_xsimd = xtl::conjunction<simd_arguments_exist,
                                                has_simd_apply<F, scalar_result_type>,
@@ -165,7 +122,7 @@ namespace xt
     /************************
      * xfunction extensions *
      ************************/
-    
+
     namespace extension
     {
 
@@ -208,6 +165,16 @@ namespace xt
         using stepper = const_stepper;
     };
 
+    template <class F, class... CT>
+    struct xcontainer_inner_types<xfunction<F, CT...>>
+    {
+        // Added indirection for MSVC 2017 bug with the operator value_type()
+        using value_type = typename meta_identity<decltype(std::declval<F>()(std::declval<xvalue_type_t<std::decay_t<CT>>>()...))>::type;
+        using reference = value_type;
+        using const_reference = value_type;
+        using size_type = common_size_type_t<std::decay_t<CT>...>;
+    };
+
     /*************
      * xfunction *
      *************/
@@ -226,25 +193,27 @@ namespace xt
     template <class F, class... CT>
     class xfunction : private xconst_iterable<xfunction<F, CT...>>,
                       public xexpression<xfunction<F, CT...>>,
+                      private xconst_accessible<xfunction<F, CT...>>,
                       public extension::xfunction_base_t<F, CT...>
     {
     public:
 
         using self_type = xfunction<F, CT...>;
+        using accessible_base = xconst_accessible<self_type>;
         using extension_base = extension::xfunction_base_t<F, CT...>;
         using expression_tag = typename extension_base::expression_tag;
         using only_scalar = all_xscalar<CT...>;
         using functor_type = typename std::remove_reference<F>::type;
         using tuple_type = std::tuple<CT...>;
 
-        // Added indirection for MSVC 2017 bug with the operator value_type()
-        using value_type = typename detail::meta_identity<decltype(std::declval<F>()(std::declval<xvalue_type_t<std::decay_t<CT>>>()...))>::type;
-        using reference = value_type;
-        using const_reference = value_type;
+        using inner_types = xcontainer_inner_types<self_type>;
+        using value_type = typename inner_types::value_type;
+        using reference = typename inner_types::reference;
+        using const_reference = typename inner_types::const_reference;
         using pointer = value_type*;
         using const_pointer = const value_type*;
-        using size_type = detail::common_size_type_t<std::decay_t<CT>...>;
-        using difference_type = detail::common_difference_type_t<std::decay_t<CT>...>;
+        using size_type = typename inner_types::size_type;
+        using difference_type = common_difference_type_t<std::decay_t<CT>...>;
         using simd_meta_getter = detail::xsimd_meta_getter<value_type, F, CT...>;
 
         using has_simd_interface = typename simd_meta_getter::use_xsimd;
@@ -303,7 +272,7 @@ namespace xt
         xfunction(xfunction&&) = default;
         xfunction& operator=(xfunction&&) = default;
 
-        size_type size() const noexcept;
+        using accessible_base::size;
         size_type dimension() const noexcept;
         const inner_shape_type& shape() const;
         layout_type layout() const noexcept;
@@ -311,17 +280,13 @@ namespace xt
         template <class... Args>
         const_reference operator()(Args... args) const;
 
-        template <class... Args>
-        const_reference at(Args... args) const;
-
         template <class... Args>
         const_reference unchecked(Args... args) const;
 
-        template <class S>
-        disable_integral_t<S, const_reference> operator[](const S& index) const;
-        template <class I>
-        const_reference operator[](std::initializer_list<I> index) const;
-        const_reference operator[](size_type i) const;
+        using accessible_base::at;
+        using accessible_base::operator[];
+        using accessible_base::periodic;
+        using accessible_base::in_bounds;
 
         template <class It>
         const_reference element(It first, It last) const;
@@ -404,6 +369,7 @@ namespace xt
         friend class xfunction_iterator<F, CT...>;
         friend class xfunction_stepper<F, CT...>;
         friend class xconst_iterable<self_type>;
+        friend class xconst_accessible<self_type>;
     };
 
     /**********************
@@ -582,15 +548,6 @@ namespace xt
      * @name Size and shape
      */
     //@{
-    /**
-     * Returns the size of the expression.
-     */
-    template <class F, class... CT>
-    inline auto xfunction<F, CT...>::size() const noexcept -> size_type
-    {
-        return compute_size(shape());
-    }
-
     /**
      * Returns the number of dimensions of the function.
      */
@@ -656,23 +613,6 @@ namespace xt
         return access_impl(std::make_index_sequence<sizeof...(CT)>(), static_cast<size_type>(args)...);
     }
 
-    /**
-     * Returns a constant reference to the element at the specified position in the expression,
-     * after dimension and bounds checking.
-     * @param args a list of indices specifying the position in the function. Indices
-     * must be unsigned integers, the number of indices should be equal to the number of dimensions
-     * of the expression.
-     * @exception std::out_of_range if the number of argument is greater than the number of dimensions
-     * or if indices are out of bounds.
-     */
-    template <class F, class... CT>
-    template <class... Args>
-    inline auto xfunction<F, CT...>::at(Args... args) const -> const_reference
-    {
-        check_access(shape(), static_cast<size_type>(args)...);
-        return this->operator()(args...);
-    }
-
     /**
      * Returns a constant reference to the element at the specified position in the expression.
      * @param args a list of indices specifying the position in the expression. Indices
@@ -681,7 +621,7 @@ namespace xt
      *
      * @warning This method is meant for performance, for expressions with a dynamic
      * number of dimensions (i.e. not known at compile time). Since it may have
-     * undefined behavior (see parameters), operator() should be prefered whenever
+     * undefined behavior (see parameters), operator() should be preferred whenever
      * it is possible.
      * @warning This method is NOT compatible with broadcasting, meaning the following
      * code has undefined behavior:
@@ -689,7 +629,7 @@ namespace xt
      * xt::xarray<double> a = {{0, 1}, {2, 3}};
      * xt::xarray<double> b = {0, 1};
      * auto fd = a + b;
-     * double res = fd.uncheked(0, 1);
+     * double res = fd.unchecked(0, 1);
      * \endcode
      */
     template <class F, class... CT>
@@ -701,27 +641,6 @@ namespace xt
         return unchecked_impl(std::make_index_sequence<sizeof...(CT)>(), static_cast<size_type>(args)...);
     }
 
-    template <class F, class... CT>
-    template <class S>
-    inline auto xfunction<F, CT...>::operator[](const S& index) const
-        -> disable_integral_t<S, const_reference>
-    {
-        return element(index.cbegin(), index.cend());
-    }
-
-    template <class F, class... CT>
-    template <class I>
-    inline auto xfunction<F, CT...>::operator[](std::initializer_list<I> index) const -> const_reference
-    {
-        return element(index.begin(), index.end());
-    }
-
-    template <class F, class... CT>
-    inline auto xfunction<F, CT...>::operator[](size_type i) const -> const_reference
-    {
-        return operator()(i);
-    }
-
     /**
      * Returns a constant reference to the element at the specified position in the function.
      * @param first iterator starting the sequence of indices
