@@ -164,23 +164,51 @@ namespace xt
          ********************/
 
         template <class T = bool>
-        struct optional_bitwise
+        class optional_bitwise
         {
+        public:
+
             using return_type = T;
             using first_argument_type = T;
             using second_argument_type = T;
             using result_type = T;
             using simd_value_type = bool;
             using simd_result_type = bool;
-            template <class T1, class T2>
-            constexpr result_type operator()(const T1& arg1, const T2& arg2) const
+            
+            template <class... Args>
+            constexpr result_type operator()(const Args&... args) const
             {
-                return (arg1 & arg2);
+                return apply_impl(args...);
             }
-            constexpr simd_result_type simd_apply(const simd_value_type& arg1,
-                                                  const simd_value_type& arg2) const
+
+            template <class... Args>
+            constexpr simd_result_type simd_apply(const Args&... args) const
             {
-                return (arg1 & arg2);
+                return simd_apply_impl(args...);
+            }
+
+        private:
+
+            constexpr result_type apply_impl() const
+            {
+                return true;
+            }
+
+            template <class U, class... Args>
+            constexpr result_type apply_impl(const U& t, const Args&... args) const
+            {
+                return t & apply_impl(args...);
+            }
+
+            constexpr simd_result_type simd_apply_impl() const
+            {
+                return simd_result_type(true);
+            }
+
+            template <class U, class... Args>
+            constexpr simd_result_type simd_apply_impl(const U& t, const Args&... args) const
+            {
+                return t & simd_apply_impl(args...);
             }
         };
     }
@@ -192,11 +220,11 @@ namespace xt
     template <class T, class B>
     auto sign(const xtl::xoptional<T, B>& e);
 
-    template <class E, XTENSOR_REQUIRE<is_xexpression<E>::value>>
+    /*template <class E, XTL_REQUIRES(is_xexpression<E>)>
     detail::value_expression_t<E> value(E&&);
 
-    template <class E, XTENSOR_REQUIRE<is_xexpression<E>::value>>
-    detail::flag_expression_t<E> has_value(E&&);
+    template <class E, XTL_REQUIRES(is_xexpression<E>)>
+    detail::flag_expression_t<E> has_value(E&&);*/
 
     template <>
     class xexpression_assigner_base<xoptional_expression_tag>
@@ -528,15 +556,16 @@ namespace xt
 
     namespace extension
     {
-        template <class F, class CT, class X>
-        class xreducer_optional : public xoptional_empty_base<xreducer<F, CT, X>>
+        template <class F, class CT, class X, class O>
+        class xreducer_optional : public xoptional_empty_base<xreducer<F, CT, X, O>>
         {
         public:
 
             using expression_tag = xoptional_expression_tag;
-            using value_expression = xreducer<F, xt::detail::value_expression_t<CT>, X>;
-            using flag_reducer = xreducer_functors<xt::detail::optional_bitwise<bool>>;
-            using flag_expression = xreducer<flag_reducer, xt::detail::flag_expression_t<CT>, X>;
+            using value_expression = xreducer<F, xt::detail::value_expression_t<CT>, X, O>;
+            using flag_reducer = xreducer_functors<xt::detail::optional_bitwise<bool>, xt::const_value<bool>>;
+            using rebound_flag_reduce_options = typename O::template rebind_t<bool>;
+            using flag_expression = xreducer<flag_reducer, xt::detail::flag_expression_t<CT>, X, rebound_flag_reduce_options>;
             using const_value_expression = value_expression;
             using const_flag_expression = flag_expression;
 
@@ -544,10 +573,10 @@ namespace xt
             const_flag_expression has_value() const;
         };
 
-        template <class F, class CT, class X>
-        struct xreducer_base_impl<xoptional_expression_tag, F, CT, X>
+        template <class F, class CT, class X, class O>
+        struct xreducer_base_impl<xoptional_expression_tag, F, CT, X, O>
         {
-            using type = xreducer_optional<F, CT, X>;
+            using type = xreducer_optional<F, CT, X, O>;
         };
     }
 
@@ -949,16 +978,21 @@ namespace xt
 
     namespace extension
     {
-        template <class F, class CT, class X>
-        inline auto xreducer_optional<F, CT, X>::value() const -> const_value_expression
+        template <class F, class CT, class X, class O>
+        inline auto xreducer_optional<F, CT, X, O>::value() const -> const_value_expression
         {
             return this->derived_cast().build_reducer(this->derived_cast().expression().value());
         }
 
-        template <class F, class CT, class X>
-        inline auto xreducer_optional<F, CT, X>::has_value() const -> const_flag_expression
+        template <class F, class CT, class X, class O>
+        inline auto xreducer_optional<F, CT, X, O>::has_value() const -> const_flag_expression
         {
-            return this->derived_cast().build_reducer(this->derived_cast().expression().has_value(), flag_reducer());
+            auto opts = this->derived_cast().options().rebind(this->derived_cast().options().initial_value.has_value(),
+                                                              this->derived_cast().options());
+
+            return this->derived_cast().build_reducer(this->derived_cast().expression().has_value(),
+                                                        make_xreducer_functor(xt::detail::optional_bitwise<bool>(),
+                                                                              xt::const_value<bool>(true)), std::move(opts));
         }
     }
 
@@ -1072,13 +1106,13 @@ namespace xt
      * value() and has_value() implementation *
      ******************************************/
 
-    template <class E, class>
+    template <class E, XTL_REQUIRES(is_xexpression<E>)>
     inline auto value(E&& e) -> detail::value_expression_t<E>
     {
         return detail::split_optional_expression<E>::value(std::forward<E>(e));
     }
 
-    template <class E, class>
+    template <class E, XTL_REQUIRES(is_xexpression<E>)>
     inline auto has_value(E&& e) -> detail::flag_expression_t<E>
     {
         return detail::split_optional_expression<E>::has_value(std::forward<E>(e));
