@@ -15,6 +15,7 @@
 
 #include <xtl/xtype_traits.hpp>
 
+#include "xaccessible.hpp"
 #include "xexpression.hpp"
 #include "xiterable.hpp"
 #include "xlayout.hpp"
@@ -71,10 +72,19 @@ namespace xt
         using stepper = xscalar_stepper<false, CT>;
     };
 
-#define DL XTENSOR_DEFAULT_LAYOUT
+    template <class CT>
+    struct xcontainer_inner_types<xscalar<CT>>
+    {
+        using value_type = std::decay_t<CT>;
+        using reference = value_type&;
+        using const_reference = const value_type&;
+        using size_type = std::size_t;
+    };
+
     template <class CT>
     class xscalar : public xexpression<xscalar<CT>>,
                     private xiterable<xscalar<CT>>,
+                    private xaccessible<xscalar<CT>>,
                     public extension::xscalar_base_t<CT>
     {
     public:
@@ -82,14 +92,16 @@ namespace xt
         using self_type = xscalar<CT>;
         using xexpression_type = std::decay_t<CT>;
         using extension_base = extension::xscalar_base_t<CT>;
+        using accessible_base = xaccessible<self_type>;
         using expression_tag = typename extension_base::expression_tag;
+        using inner_types = xcontainer_inner_types<self_type>;
 
-        using value_type = std::decay_t<CT>;
-        using reference = value_type&;
-        using const_reference = const value_type&;
+        using value_type = typename inner_types::value_type;
+        using reference = typename inner_types::reference;
+        using const_reference = typename inner_types::const_reference;
         using pointer = value_type*;
         using const_pointer = const value_type*;
-        using size_type = std::size_t;
+        using size_type = typename inner_types::size_type;
         using difference_type = std::ptrdiff_t;
         using simd_value_type = xsimd::simd_type<value_type>;
 
@@ -138,33 +150,25 @@ namespace xt
         operator const value_type&() const noexcept;
 
         size_type size() const noexcept;
-        size_type dimension() const noexcept;
         const shape_type& shape() const noexcept;
         layout_type layout() const noexcept;
+        using accessible_base::dimension;
+        using accessible_base::shape;
 
         template <class... Args>
         reference operator()(Args...) noexcept;
         template <class... Args>
-        reference at(Args...);
-        template <class... Args>
         reference unchecked(Args...) noexcept;
-        template <class S>
-        disable_integral_t<S, reference> operator[](const S&) noexcept;
-        template <class I>
-        reference operator[](std::initializer_list<I>) noexcept;
-        reference operator[](size_type) noexcept;
 
         template <class... Args>
         const_reference operator()(Args...) const noexcept;
         template <class... Args>
-        const_reference at(Args...) const;
-        template <class... Args>
         const_reference unchecked(Args...) const noexcept;
-        template <class S>
-        disable_integral_t<S, const_reference> operator[](const S&) const noexcept;
-        template <class I>
-        const_reference operator[](std::initializer_list<I>) const noexcept;
-        const_reference operator[](size_type) const noexcept;
+
+        using accessible_base::at;
+        using accessible_base::operator[];
+        using accessible_base::periodic;
+        using accessible_base::in_bounds;
 
         template <class It>
         reference element(It, It) noexcept;
@@ -181,61 +185,61 @@ namespace xt
         template <class S>
         bool has_linear_assign(const S& strides) const noexcept;
 
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         iterator begin() noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         iterator end() noexcept;
 
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_iterator begin() const noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_iterator end() const noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_iterator cbegin() const noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_iterator cend() const noexcept;
 
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         reverse_iterator rbegin() noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         reverse_iterator rend() noexcept;
 
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_iterator rbegin() const noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_iterator rend() const noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_iterator crbegin() const noexcept;
-        template <layout_type L = DL>
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_iterator crend() const noexcept;
 
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         broadcast_iterator<S, L> begin(const S& shape) noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         broadcast_iterator<S, L> end(const S& shape) noexcept;
 
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_broadcast_iterator<S, L> begin(const S& shape) const noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_broadcast_iterator<S, L> end(const S& shape) const noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_broadcast_iterator<S, L> cbegin(const S& shape) const noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_broadcast_iterator<S, L> cend(const S& shape) const noexcept;
 
 
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         reverse_broadcast_iterator<S, L> rbegin(const S& shape) noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         reverse_broadcast_iterator<S, L> rend(const S& shape) noexcept;
 
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_broadcast_iterator<S, L> rbegin(const S& shape) const noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_broadcast_iterator<S, L> rend(const S& shape) const noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_broadcast_iterator<S, L> crbegin(const S& shape) const noexcept;
-        template <class S, layout_type L = DL>
+        template <class S, layout_type L = XTENSOR_DEFAULT_TRAVERSAL>
         const_reverse_broadcast_iterator<S, L> crend(const S& shape) const noexcept;
 
         iterator storage_begin() noexcept;
@@ -286,8 +290,9 @@ namespace xt
 
         friend class xconst_iterable<self_type>;
         friend class xiterable<self_type>;
+        friend class xaccessible<self_type>;
+        friend class xconst_accessible<self_type>;
     };
-#undef DL
 
     namespace detail
     {
@@ -444,12 +449,12 @@ namespace xt
                    const xdummy_iterator<is_const, CT>& rhs) noexcept;
 
     template <class T>
-    struct is_xdummy_iterator : std::false_type
+    struct is_not_xdummy_iterator : std::true_type
     {
     };
 
     template <bool is_const, class CT>
-    struct is_xdummy_iterator<xdummy_iterator<is_const, CT>> : std::true_type
+    struct is_not_xdummy_iterator<xdummy_iterator<is_const, CT>> : std::false_type
     {
     };
 
@@ -519,12 +524,6 @@ namespace xt
         return 1;
     }
 
-    template <class CT>
-    inline auto xscalar<CT>::dimension() const noexcept -> size_type
-    {
-        return 0;
-    }
-
     template <class CT>
     inline auto xscalar<CT>::shape() const noexcept -> const shape_type&
     {
@@ -546,14 +545,6 @@ namespace xt
         return m_value;
     }
 
-    template <class CT>
-    template <class... Args>
-    inline auto xscalar<CT>::at(Args... args) -> reference
-    {
-        check_dimension(shape(), args...);
-        return this->operator()(args...);
-    }
-
     template <class CT>
     template <class... Args>
     inline auto xscalar<CT>::unchecked(Args...) noexcept -> reference
@@ -561,28 +552,6 @@ namespace xt
         return m_value;
     }
 
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::operator[](const S&) noexcept
-        -> disable_integral_t<S, reference>
-    {
-        return m_value;
-    }
-
-    template <class CT>
-    template <class I>
-    inline auto xscalar<CT>::operator[](std::initializer_list<I>) noexcept
-        -> reference
-    {
-        return m_value;
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::operator[](size_type) noexcept -> reference
-    {
-        return m_value;
-    }
-
     template <class CT>
     template <class... Args>
     inline auto xscalar<CT>::operator()(Args...) const noexcept -> const_reference
@@ -591,14 +560,6 @@ namespace xt
         return m_value;
     }
 
-    template <class CT>
-    template <class... Args>
-    inline auto xscalar<CT>::at(Args... args) const -> const_reference
-    {
-        check_dimension(shape(), args...);
-        return this->operator()(args...);
-    }
-
     template <class CT>
     template <class... Args>
     inline auto xscalar<CT>::unchecked(Args...) const noexcept -> const_reference
@@ -606,28 +567,6 @@ namespace xt
         return m_value;
     }
 
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::operator[](const S&) const noexcept
-        -> disable_integral_t<S, const_reference>
-    {
-        return m_value;
-    }
-
-    template <class CT>
-    template <class I>
-    inline auto xscalar<CT>::operator[](std::initializer_list<I>) const noexcept
-        -> const_reference
-    {
-        return m_value;
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::operator[](size_type) const noexcept -> const_reference
-    {
-        return m_value;
-    }
-
     template <class CT>
     template <class It>
     inline auto xscalar<CT>::element(It, It) noexcept -> reference
@@ -714,14 +653,14 @@ namespace xt
     template <layout_type L>
     inline auto xscalar<CT>::rbegin() noexcept -> reverse_iterator
     {
-        return reverse_storage_iterator(end());
+        return reverse_iterator(end());
     }
 
     template <class CT>
     template <layout_type L>
     inline auto xscalar<CT>::rend() noexcept -> reverse_iterator
     {
-        return reverse_storage_iterator(begin());
+        return reverse_iterator(begin());
     }
 
     template <class CT>
@@ -742,14 +681,14 @@ namespace xt
     template <layout_type L>
     inline auto xscalar<CT>::crbegin() const noexcept -> const_reverse_iterator
     {
-        return const_reverse_storage_iterator(cend());
+        return const_reverse_iterator(cend());
     }
 
     template <class CT>
     template <layout_type L>
     inline auto xscalar<CT>::crend() const noexcept -> const_reverse_iterator
     {
-        return const_reverse_storage_iterator(cbegin());
+        return const_reverse_iterator(cbegin());
     }
 
     /*****************************
