@@ -14,6 +14,7 @@
 #include <numeric>
 
 #include "xexception.hpp"
+#include "xshape.hpp"
 #include "xtensor_forward.hpp"
 
 namespace xt
@@ -61,10 +62,13 @@ namespace xt
      *****************/
 
     template <class S>
-    S unravel_from_strides(typename S::value_type index, const S& strides, layout_type l);
+    S unravel_from_strides(typename S::value_type index, const S& strides, layout_type l=layout_type::row_major);
 
     template <class S>
-    get_strides_t<S> unravel_index(typename S::value_type index, const S& shape, layout_type l);
+    get_strides_t<S> unravel_index(typename S::value_type index, const S& shape, layout_type l=layout_type::row_major);
+
+    template <class S, class T>
+    std::vector<get_strides_t<S>> unravel_indices(const T& indices, const S& shape, layout_type l=layout_type::row_major);
 
     /***********************
      * broadcast functions *
@@ -83,34 +87,76 @@ namespace xt
     template <layout_type L>
     struct check_strides_overlap;
 
+    /**********************************
+     * check bounds, without throwing *
+     **********************************/
+
+    template <class S, class... Args>
+    bool in_bounds(const S& shape, Args&... args);
+
+    /********************************
+     * apply periodicity to indices *
+     *******************************/
+
+    template <class S, class... Args>
+    void normalize_periodic(const S& shape, Args&... args);
+
     /********************************************
      * utility functions for strided containers *
      ********************************************/
 
-    template <class C, class It>
-    It strided_data_end(const C& c, It end, layout_type l)
+    template <class C, class It, class size_type>
+    It strided_data_end(const C& c, It begin, layout_type l, size_type offset)
     {
         using difference_type = typename std::iterator_traits<It>::difference_type;
         if (c.dimension() == 0)
         {
-            return end;
+            ++begin;
         }
         else
         {
-            auto leading_stride = (l == layout_type::row_major ? c.strides().back() : c.strides().front());
-            return end + difference_type(leading_stride - 1);
+            for (std::size_t i = 0; i != c.dimension(); ++i)
+            {
+                begin += c.strides()[i] * difference_type(c.shape()[i] - 1);
+            }
+            if (l == layout_type::row_major)
+            {
+                begin += c.strides().back();
+            }
+            else
+            {
+                if (offset == 0)
+                {
+                    begin += c.strides().front();
+                }
+            }
         }
+        return begin;
     }
 
     /******************
      * Implementation *
      ******************/
 
+    namespace detail
+    {
+        template <class shape_type>
+        inline std::size_t compute_size_impl(const shape_type& shape, std::true_type /* is signed */) {
+            using size_type = std::decay_t<typename shape_type::value_type>;
+            return static_cast<std::size_t>(std::abs(std::accumulate(shape.cbegin(), shape.cend(), size_type(1), std::multiplies<size_type>())));
+        }
+
+        template <class shape_type>
+        inline std::size_t compute_size_impl(const shape_type& shape, std::false_type /* is not signed */) {
+            using size_type = std::decay_t<typename shape_type::value_type>;
+            return static_cast<std::size_t>(std::accumulate(shape.cbegin(), shape.cend(), size_type(1), std::multiplies<size_type>()));
+        }
+    }
+
     template <class shape_type>
     inline std::size_t compute_size(const shape_type& shape) noexcept
     {
-        using size_type = std::decay_t<typename shape_type::value_type>;
-        return static_cast<std::size_t>(std::accumulate(shape.cbegin(), shape.cend(), size_type(1), std::multiplies<size_type>()));
+        return detail::compute_size_impl(shape, std::is_signed<std::decay_t<typename std::decay_t<shape_type>::value_type>>());
     }
 
     namespace detail
@@ -265,7 +311,7 @@ namespace xt
         inline std::size_t compute_strides(const shape_type& shape, layout_type l,
                                            strides_type& strides, bs_ptr bs)
         {
-            using strides_value_type = std::decay_t<decltype(strides[0])>;
+            using strides_value_type = typename std::decay_t<strides_type>::value_type;
             strides_value_type data_size = 1;
             if (L == layout_type::row_major || l == layout_type::row_major)
             {
@@ -303,8 +349,10 @@ namespace xt
         return detail::compute_strides<L>(shape, l, strides, &backstrides);
     }
 
+    // base_stride is the stride value used when shape is 1
     template <class shape_type, class strides_type>
-    inline bool do_strides_match(const shape_type& shape, const strides_type& strides, layout_type l)
+    inline bool do_strides_match(const shape_type& shape, const strides_type& strides,
+                                 layout_type l, typename strides_type::value_type base_stride)
     {
         using value_type = typename strides_type::value_type;
         value_type data_size = 1;
@@ -312,7 +360,7 @@ namespace xt
         {
             for (std::size_t i = strides.size(); i != 0; --i)
             {
-                if ((shape[i - 1] == 1 && strides[i - 1] != 0) || (shape[i - 1] != 1 && strides[i - 1] != data_size))
+                if ((shape[i - 1] == 1 && strides[i - 1] != base_stride) || (shape[i - 1] != 1 && strides[i - 1] != data_size))
                 {
                     return false;
                 }
@@ -324,7 +372,7 @@ namespace xt
         {
             for (std::size_t i = 0; i < strides.size(); ++i)
             {
-                if ((shape[i] != 1 && strides[i] != data_size) || (shape[i] == 1 && strides[i] != 0))
+                if ((shape[i] != 1 && strides[i] != data_size) || (shape[i] == 1 && strides[i] != base_stride))
                 {
                     return false;
                 }
@@ -399,15 +447,49 @@ namespace xt
         return detail::unravel_noexcept(index, strides, l);
     }
 
+    template <class S>
+    inline get_strides_t<S> ravel_from_strides(const S& index, const S& strides)
+    {
+        return element_offset<get_strides_t<S>>(strides, index.begin(), index.end());
+    }
+
     template <class S>
     inline get_strides_t<S> unravel_index(typename S::value_type index, const S& shape, layout_type l)
     {
-        get_strides_t<S> strides = xtl::make_sequence<get_strides_t<S>>(shape.size(), 0);
-        using strides_value_type = std::decay_t<decltype(strides[0])>;
+        using strides_type = get_strides_t<S>;
+        using strides_value_type = typename strides_type::value_type;
+        strides_type strides = xtl::make_sequence<strides_type>(shape.size(), 0);
         compute_strides(shape, l, strides);
         return unravel_from_strides(static_cast<strides_value_type>(index), strides, l);
     }
 
+    template <class S, class T>
+    inline std::vector<get_strides_t<S>> unravel_indices(const T& idx, const S& shape, layout_type l)
+    {
+        using strides_type = get_strides_t<S>;
+        using strides_value_type = typename strides_type::value_type;
+        strides_type strides = xtl::make_sequence<strides_type>(shape.size(), 0);
+        compute_strides(shape, l, strides);
+        std::vector<get_strides_t<S>> out(idx.size());
+        auto out_iter = out.begin();
+        auto idx_iter = idx.begin();
+        for ( ; out_iter != out.end(); ++out_iter, ++idx_iter )
+        {
+            *out_iter = unravel_from_strides(static_cast<strides_value_type>(*idx_iter), strides, l);
+        }
+        return out;
+    }
+
+    template <class S>
+    inline get_value_type<S> ravel_index(const S& index, const S& shape, layout_type l)
+    {
+        using strides_type = get_strides_t<S>;
+        using strides_value_type = typename strides_type::value_type;
+        strides_type strides = xtl::make_sequence<strides_type>(shape.size(), 0);
+        compute_strides(shape, l, strides);
+        return ravel_from_strides(static_cast<strides_value_type>(index), strides);
+    }
+
     template <class S1, class S2>
     inline bool broadcast_shape(const S1& input, S2& output)
     {
@@ -430,7 +512,7 @@ namespace xt
             {
                 output[output_index - 1] = static_cast<value_type>(input[input_index - 1]);
             }
-            // Second case: output has been initialized to 1. Broacast is trivial
+            // Second case: output has been initialized to 1. Broadcast is trivial
             // only if input is 1 to.
             else if (output[output_index - 1] == 1)
             {
@@ -500,7 +582,7 @@ namespace xt
             size_type index = 0;
 
             // This check is necessary as column major "broadcasting" is still
-            // peformed in a row major fashion
+            // performed in a row major fashion
             if (s1.size() != s2.size())
                 return 0;
 
@@ -516,6 +598,64 @@ namespace xt
             return index;
         }
     };
+
+    namespace detail
+    {
+        template <class S, std::size_t dim>
+        inline bool check_in_bounds_impl(const S&)
+        {
+            return true;
+        }
+
+        template <class S, std::size_t dim, class T, class... Args>
+        inline bool check_in_bounds_impl(const S& shape, T& arg, Args&... args)
+        {
+            if (sizeof...(Args) + 1 > shape.size())
+            {
+                return check_in_bounds_impl<S, dim>(shape, args...);
+            }
+            else
+            {
+                return arg >= T(0) && arg < static_cast<T>(shape[dim]) && check_in_bounds_impl<S, dim + 1>(shape, args...);
+            }
+        }
+    }
+
+    template <class S, class... Args>
+    inline bool check_in_bounds(const S& shape, Args&... args)
+    {
+        return detail::check_in_bounds_impl<S, 0>(shape, args...);
+    }
+
+    namespace detail
+    {
+        template <class S, std::size_t dim>
+        inline void normalize_periodic_impl(const S&)
+        {
+        }
+
+        template <class S, std::size_t dim, class T, class... Args>
+        inline void normalize_periodic_impl(const S& shape, T& arg, Args&... args)
+        {
+            if (sizeof...(Args) + 1 > shape.size())
+            {
+                normalize_periodic_impl<S, dim>(shape, args...);
+            }
+            else
+            {
+                T n = static_cast<T>(shape[dim]);
+                arg = (n + (arg%n)) % n;
+                normalize_periodic_impl<S, dim + 1>(shape, args...);
+            }
+        }
+    }
+
+    template <class S, class... Args>
+    inline void normalize_periodic(const S& shape, Args&... args)
+    {
+        check_dimension(shape, args...);
+        detail::normalize_periodic_impl<S, 0>(shape, args...);
+    }
 }
 
 #endif
